#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Classify forums converted from XML to stemmed bag of words with metadata.

:author: `David King <mailto:david.king@open.ac.uk>`_
:project: eSTEeM - Measuring Qualification Effects framework
:copyright: `2016 The Open University <http://www.open.ac.uk>`_
:licence: `GPL v3 <http://www.gnu.org/licenses/>`_
"""


from collections import Counter
from datetime import date
import html
import glob
import os
import pickle
import re
import sys
import xml.etree.ElementTree as ET

import nltk

STEMMER = nltk.LancasterStemmer()

STRIP_TAGS = re.compile(r'<[^>]*>').sub


def is_current(post):
    """Return True if post is latest version of that post in the forum."""
    return True if post.find('oldversion').text == '0' else False


def load_taxonomy(taxon_set, classifiers_folder):
    """Return taxonomy to classify forum posts."""
    taxon_file = os.path.join(classifiers_folder, taxon_set + '.pkl')
    with open(taxon_file, 'rb') as fin:
        return pickle.load(fin)


def set_results_folder(results_folder):
    """Ensure we have somewhere to write."""
    if not os.path.exists(results_folder):
        os.mkdir(results_folder)


def get_forum_data(source_folder):
    """Return forum name and parsed XML from each file in source folder."""
    xml_files = glob.iglob(os.path.join(source_folder, '*.xml'))
    for xml_file in xml_files:
        forum_name = os.path.basename(xml_file)[:-4]
        tree = ET.parse(xml_file)
        root = tree.getroot()
        posts = root.iter('post')
        yield (forum_name, posts)


def classify_posts(source_folder,
                   taxon_set='bloom',
                   classifiers_folder='../classifiers',
                   results_folder='../posts-classified/'):
    """Apply classification to forum posts XML files in source folder.."""
    print('Classifying forum posts in {}'.format(source_folder),
          flush=True)
    set_results_folder(results_folder)
    taxonomy = load_taxonomy(taxon_set, classifiers_folder)
    for forum_name, posts in get_forum_data(source_folder):
        print('Processing {}'.format(forum_name), flush=True)
        output_file = os.path.join(results_folder, forum_name + '.pkl')
        with open(output_file, 'wb') as fout:
            pickle.dump(process_posts(posts, forum_name, taxon_set, taxonomy),
                        fout, pickle.HIGHEST_PROTOCOL)


def process_posts(posts, forum_name, taxon_set, taxonomy):
    """Return stemmed message and metadata for forum posts."""
    categorised_posts = []
    for post in posts:
        if is_current(post):
            categorised_post = {}
            categorised_post['postid'] = post.get('id')
            categorised_post['userid'] = post.find('userid').text
            created = post.find('created')
            post_date = date.fromtimestamp(int(created.text))
            categorised_post['created'] = post_date
            categorised_post['forum'] = forum_name
            message_text = post.find('message').text
            categorised_post['surface'] = message_text
            tokenised_message = tokenise_message(message_text)
            categorised_post['tokens'] = tokenised_message
            categorised_post[taxon_set] = categorise_message(tokenised_message,
                                                             taxonomy)
            categorised_posts.append(categorised_post)
    return categorised_posts


def tokenise_message(message_text):
    """Return unique stemmed tokens of message text."""
    message_unescaped = html.unescape(message_text)
    message_tagless = STRIP_TAGS(' ', message_unescaped)
    return nltk.word_tokenize(message_tagless)


def categorise_message(post_tokens, taxonomy):
    """Return count of matched categorisable terms in forum post."""
    stemmed_tokens = [STEMMER.stem(token) for token in post_tokens]
    stemmed_counts = Counter(stemmed_tokens)
    cat_counts = {key: 0 for key in taxonomy.keys()}  # category counts
    for key, values in taxonomy.items():
        for value in values:
            if value in stemmed_counts:
                cat_counts[key] += stemmed_counts[value]
    return cat_counts


if __name__ == '__main__':
    sys.exit(classify_posts('../forums/'))
    # sys.exit(classify_posts('../forums/',
                            # taxon_set='solo',
                            # results_folder='../posts-classified-solo/'))
