#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Draw line chart of classified and filtered posts made during presentation.

:author: `David King <mailto:david.king@open.ac.uk>`_
:project: eSTEeM - Measuring Qualification Effects framework
:copyright: `2016 The Open University <http://www.open.ac.uk>`_
:licence: `GPL v3 <http://www.gnu.org/licenses/>`_
"""

import csv
from datetime import date
import glob
import os
import sys

from matplotlib import pyplot as plt


def draw_posts(source_folder, images_folder='../images/'):
    """Submit for processing forums in source folder."""
    print('Processing data files in {}'.format(source_folder),
          flush=True)
    set_results_folder(images_folder)
    for forum_filename, posts_data in get_posts_data(source_folder):
        draw_linechart(forum_filename, posts_data, images_folder)


def set_results_folder(results_folder):
    """Ensure we have somewhere to write."""
    if not os.path.exists(results_folder):
        os.mkdir(results_folder)


def get_posts_data(source_folder):
    """Return forum name and data from each file in source folder."""
    daily_files = glob.iglob(os.path.join(source_folder, '*_daily_counts.csv'))
    filtered_files = glob.iglob(os.path.join(source_folder, '*_filtered_counts.csv'))
    for daily_file, filtered_file in zip(daily_files, filtered_files):
        forum_filename = os.path.basename(daily_file)[:-17]
        print('Processing {}'.format(forum_filename), flush=True)
        daily_counts = get_posts_counts(daily_file)
        filtered_counts = get_posts_counts(filtered_file)
        merged_posts_counts = merge_posts_counts(daily_counts, filtered_counts)
        yield forum_filename, merged_posts_counts


def get_posts_counts(counts_file):
    """Return dates with posts and number of posts on each date."""
    with open(counts_file, 'rt', encoding='utf-8', newline='\n') as fin:
        csv_counts = csv.reader(fin)
        dates, counts = zip(*csv_counts)
        dates = dates[1:]  # remove header
        counts = counts[1:]  # remove header
    return dict(zip(dates, counts))


def merge_posts_counts(daily_counts, filtered_counts):
    """Return merged dict of date with both filtered and classified counts."""
    merged_counts = {}
    for k in daily_counts:
        merged_counts[k] = (daily_counts[k], filtered_counts.get(k, 0))
    if not merged_counts:  # if empty forum
        for k in filtered_counts:
            merged_counts[k] = (0, 0)
    return merged_counts


def draw_linechart(forum_filename, posts_data, images_folder):
    """Plot counts by date as a line chart to png."""
    outer_fig, inner_ax = plt.subplots()
    dates, daily_counts, filtered_counts = prepare_plot_data(posts_data)
    inner_ax.plot(dates, daily_counts, '.', color='blue', linestyle='solid')
    inner_ax.plot(dates, filtered_counts, '.', color='red', linestyle='solid')
    inner_ax.grid()
    inner_ax.set_title('{}'.format(forum_filename))
    inner_ax.set_ylabel('Number of posts')
    outer_fig.autofmt_xdate()
    png_file = os.path.join(images_folder, forum_filename + '.png')
    plt.savefig(png_file)


def prepare_plot_data(merged_counts):
    """Return separate lists of data for plotting."""
    sorted_counts = sorted(list(merged_counts.items()))
    raw_dates, counts = zip(*sorted_counts)
    dates = [convert_date(raw_date) for raw_date in raw_dates]
    daily_counts, filtered_counts = zip(*counts)
    return dates, daily_counts, filtered_counts


def convert_date(raw_date):
    """Return date string as Python datetime."""
    date_year, date_month, date_day = raw_date.split('-')
    return date(int(date_year), int(date_month), int(date_day))


if __name__ == '__main__':
    sys.exit(draw_posts('../data/'))
    # sys.exit(draw_posts('../data-test/', images_folder='../images-test/'))
