#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Extract forum archives from Moodle course backup dump.

``.mbz`` files are actually tar files with gzip used for compression.
Therefore, to extract the forum xml file from the course backup
we can use Python's built-in tarfile module.

Forum files are all named ``forumng.xml``. Therefore, we loop through
the ``.mbz`` file looking for files of that name. When found,
we *extract* the file. There may be several files in the
backup, dependent on the number of forums in the course. One file for
each forum.

Python's tarfile does not let you rename or redirect files when they
are extracted. Therefore. we first extract the file, to the location and
using the name specified in the backup. Then we copy the extracted file
to a new location based on the target output ``forums_folder``, giving
the new copy of the file a name derived from the presentation code in the
name of the ``.mbz`` backup file and the individual forum name in the
``name`` field of the xml data. At the end of the script, we delete the
original extracted files, leaving only the copies for subsequent use.

:author: `David King <mailto:david.king@open.ac.uk>`_
:project: eSTEeM - Measuring Qualification Effects framework
:copyright: `2016 The Open University <http://www.open.ac.uk>`_
:licence: `GPL v3 <http://www.gnu.org/licenses/>`_
"""

import glob
import os
import re
import shutil
from string import punctuation
import sys
import tarfile
import xml.etree.ElementTree as ET

FIND_PRESENTATION = re.compile(r'''
(?P<presentation>[a-z]+\d{3}-\d{2}[a-m])
''', re.VERBOSE).search

STRIP_PUNCTUATION = str.maketrans(punctuation,
                                  ' ' * len(punctuation),
                                  punctuation)


def get_forum_xml(source_folder, forums_folder='../forums'):
    """Extract forums from Moodle backups in ``source_folder``."""
    print('\nProcessing folder {}'.format(source_folder), flush=True)
    set_results_folder(forums_folder)
    mbz_files = glob.iglob(os.path.join(source_folder, '*.mbz'))
    for mbz_file in mbz_files:
        print('\nProcessing file {}'.format(mbz_file), flush=True)
        extract_forum_archives(mbz_file, forums_folder)
    print('\nRemoving temporary extraction folder', flush=True)
    shutil.rmtree('activities')


def set_results_folder(results_folder):
    """Ensure we have somewhere to write."""
    if not os.path.exists(results_folder):
        os.mkdir(results_folder)


def extract_forum_archives(mbz_file, forums_folder):
    """Extract all forum files in ``mbz_file`` to ``forums_folder``."""
    presentation = FIND_PRESENTATION(mbz_file).group('presentation').upper()
    with tarfile.open(mbz_file, 'r:gz') as tar:
        for tarinfo in tar:
            if 'forumng.xml' in tarinfo.name:
                print('Reading {}'.format(tarinfo.name), flush=True)
                tar.extract(tarinfo)
                with open(tarinfo.name, 'rt',
                          encoding='utf-8', newline='\n') as fin:
                    tree = ET.parse(fin)
                    forum_filename = get_forum_filename(presentation, tree)
                print('Renaming {}'.format(forum_filename), flush=True)
                forum_filepath = os.path.join(forums_folder, forum_filename)
                os.replace(tarinfo.name, forum_filepath)


def get_forum_filename(presentation, forum_xml):
    """Return forum filename from ``presentation`` and name in ``forum_xml``."""
    root = forum_xml.getroot()
    forum_name = root.findtext('./forumng/name').translate(STRIP_PUNCTUATION)
    return '{} {}.xml'.format(presentation, forum_name)


if __name__ == '__main__':
    sys.exit(get_forum_xml('../mbz'))
