#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Search VLE forum posts based on use of external references to exclude.

:author: `David King <mailto:david.king@open.ac.uk>`_
:project: eSTEeM - Measuring Qualification Effects framework
:copyright: `2016 The Open University <http://www.open.ac.uk>`_
:licence: `GPL v3 <http://www.gnu.org/licenses/>`_
"""

from datetime import date
import glob
import os
import re
import sys
import xml.etree.ElementTree as ET

FIND_CITATION = re.compile(r'\[\d+\]').search
FIND_HTTP = re.compile(r'https?\:').search
FIND_REFERENCES = re.compile(r'\breferences\b', re.IGNORECASE).search
FIND_YEAR_REF = re.compile(r'\(\d{4}\)').search

PAGE_HEADER = '''<!DOCTYPE html>
<html>
<head>
<meta charset="utf-8">
<title>{}</title>
</head>
<body>
'''

PAGE_FOOTER = '''</body>
</html>
'''

TABLE_HEADER = '''<table>
<tr>
<th>Forum</th>
<th colspan=2>with</th>
<th colspan=2>without</th>
<th>Total posts</th>
</tr>
'''

TABLE_ROW = '''<tr>
<td><a href="{}.html" target="_blank">{}</a></td>
<td align=right>{}</td>
<td align=right>{}%</td>
<td align=right>{}</td>
<td align=right>{}%</td>
<td align=right>{}</td>
</tr>
'''


def is_current(post):
    """Return True if post is latest version of that post in the forum."""
    return True if post.find('oldversion').text == '0' else False


def has_link(post):
    """Return True if post has an indicator of external link."""
    message_text = post.find('message').text
    return True if (FIND_HTTP(message_text) or
                    FIND_REFERENCES(message_text) or
                    FIND_YEAR_REF(message_text) or
                    FIND_CITATION(message_text)) else False


def get_forum_data(source_folder):
    """Return forum name and parsed XML from each file in source folder."""
    xml_files = glob.iglob(os.path.join(source_folder, '*.xml'))
    for xml_file in xml_files:
        forum_name = os.path.basename(xml_file)[:-4]
        tree = ET.parse(xml_file)
        root = tree.getroot()
        posts = root.iter('post')
        yield (forum_name, posts)


def set_results_folder(results_folder):
    """Ensure we have somewhere to write."""
    if not os.path.exists(results_folder):
        os.mkdir(results_folder)


def filterfalse_posts(source_folder, reports_folder='../reports-filterfalse/'):
    """Submit for processing all .xml files in source folder."""
    print('Processing forum posts files in {}'.format(source_folder),
          flush=True)
    set_results_folder(reports_folder)
    results = []
    for forum_name, posts in get_forum_data(source_folder):
        print('Processing {}'.format(forum_name), flush=True)
        results.append(filterfalse_forum(forum_name, posts, reports_folder))
    write_index(source_folder, results, reports_folder)


def filterfalse_forum(forum_name, posts, reports_folder):
    """Write page for forum posts that have links, to results folder."""
    output_file = os.path.join(reports_folder, forum_name + '.html')
    with open(output_file, 'wt', encoding='utf-8', newline='\n') as fout:
        fout.write(PAGE_HEADER.format(forum_name))
        link_count = no_link_count = 0
        for post in posts:
            if is_current(post):
                if has_link(post):
                    link_count += 1
                else:
                    postid = post.get('id')
                    fout.write('<p><b>Post:</b> {}</p>\n'.format(postid))
                    userid = post.find('userid')
                    fout.write('<p><b>UserID:</b> {}</p>\n'.format(userid.text))
                    created = post.find('created')
                    created_date = date.fromtimestamp(int(created.text))
                    fout.write('<p><b>Created:</b> {}</p>\n'.format(created_date))
                    message_text = post.find('message').text
                    fout.write('<p><b>Message:</b></p>\n{}\n<hr>\n'.format(message_text))
                    no_link_count += 1
        fout.write(PAGE_FOOTER)
    return forum_name, link_count, no_link_count


def write_index(source_folder, results, reports_folder):
    """Write summary/index file to results folder."""
    with open(reports_folder + 'index.html', 'wt',
              encoding='utf-8', newline='\n') as fout:
        fout.write(PAGE_HEADER.format('Summary for ' + source_folder))
        fout.write(TABLE_HEADER)
        for forum_name, posts_with, posts_without in results:
            posts_total = posts_with + posts_without
            posts_with_pct = round((posts_with / posts_total) * 100) if posts_with else 0
            posts_without_pct = 100 - posts_with_pct
            fout.write(TABLE_ROW.format(forum_name, forum_name,
                                        posts_with, posts_with_pct,
                                        posts_without, posts_without_pct,
                                        posts_total))
        fout.write('</table>\n' + PAGE_FOOTER)


if __name__ == '__main__':
    sys.exit(filterfalse_posts('../forums/'))
