#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Produce stemmed set of Bloom keywords to classify messages.

:author: `David King <mailto:david.king@open.ac.uk>`_
:project: eSTEeM - Measuring Qualification Effects framework
:copyright: `2016 The Open University <http://www.open.ac.uk>`_
:licence: `GPL v3 <http://www.gnu.org/licenses/>`_
"""


from configparser import ConfigParser
import os
import pickle
import sys

import nltk

STEMMER = nltk.LancasterStemmer()


def prepare_taxonomy(taxon_set,
                     classifiers_folder='../classifiers',
                     results_folder='../classifiers'):
    """Save stemmed version of taxonomy."""
    set_results_folder(results_folder)
    taxonomy = get_taxonomy(taxon_set, classifiers_folder)
    write_taxonomy(taxon_set, taxonomy, results_folder)
    pickle_taxonomy(taxon_set, taxonomy, results_folder)


def set_results_folder(results_folder):
    """Ensure we have somewhere to write."""
    if not os.path.exists(results_folder):
        os.mkdir(results_folder)


def get_taxonomy(taxon_set, classifiers_folder):
    """Return stemmed version of taxonomy."""
    cfg = ConfigParser(allow_no_value=True, strict=False)
    cfg.read(os.path.join(classifiers_folder, taxon_set + '.ini'))
    taxonomy = {}
    for key in cfg.sections():
        taxonomy[key] = set(STEMMER.stem(value[0]) for value in cfg[key].items())
    return taxonomy


def write_taxonomy(taxon_set, taxonomy, results_folder):
    """Write ``taxonomy`` to ``taxon_set`` txt file in ``results_folder``."""
    with open(os.path.join(results_folder, taxon_set + '_stemmed.txt'),
              'wt', encoding='utf-8', newline='\n') as fout:
        fout.write('Stemmed words prepared for {} taxonomy\n'.format(taxon_set))
        for key in taxonomy:
            fout.write('\n{}:\n'.format(key))
            for value in taxonomy[key]:
                fout.write('{}\n'.format(value))


def pickle_taxonomy(taxon_set, taxonomy, results_folder):
    """Write ``taxonomy`` to ``taxon_set`` pickle file in ``results_folder``."""
    with open(os.path.join(results_folder, taxon_set + '.pkl'),
              'wb') as fout:
        pickle.dump(taxonomy, fout, pickle.HIGHEST_PROTOCOL)


if __name__ == '__main__':
    sys.exit(prepare_taxonomy('bloom'))
    # sys.exit(prepare_taxonomy('solo'))
