#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Suggest keywords using tf:idf for new classification criteria.

:author: `David King <mailto:david.king@open.ac.uk>`_
:project: eSTEeM - Measuring Qualification Effects framework
:copyright: `2016 The Open University <http://www.open.ac.uk>`_
:licence: `GPL v3 <http://www.gnu.org/licenses/>`_
"""

import csv
import glob
import html
import math
import os
import re
import sys
import xml.etree.ElementTree as ET

from collections import Counter
from operator import itemgetter

EXTRACT_WORDS = re.compile(r'\w+').findall
STRIP_TAGS = re.compile(r'<[^>]*>').sub

PAGE_HEADER = '''<!DOCTYPE html>
<html>
<head>
<meta charset="utf-8">
<title>{}</title>
</head>
<body>
'''

PAGE_FOOTER = '''</body>
</html>
'''

TABLE_HEADER = '''<table>
<tr>
<th>Word</th>
<th>Score</th>
</tr>
'''

TABLE_ROW = '''<tr>
<td align=left>{:s}</td>
<td align=right>{:.8f}</td>
</tr>
'''

# from http://ir.dcs.gla.ac.uk/resources/linguistic_utils/stop_words
STOP_FILE = 'stop_words.txt'


def is_current(post):
    """Return True if post is latest version of that post in the forum."""
    return True if post.find('oldversion').text == '0' else False


def set_results_folder(results_folder):
    """Ensure we have somewhere to write."""
    if not os.path.exists(results_folder):
        os.mkdir(results_folder)


def get_stop_words(stop_file):
    """Return list of words defined in stop_file."""
    with open(stop_file, 'rt', encoding='utf-8', newline='\n') as fin:
        return fin.read().split('\n')


def suggest_keywords(source_folder,
                     data_folder='../data/',
                     reports_folder='../reports/'):
    """Submit for processing all .xml files in source folder."""
    print('Processing forum posts files in {}'.format(source_folder),
          flush=True)
    set_results_folder(data_folder)
    set_results_folder(reports_folder)
    forum_names = []
    forum_texts = []
    word_counts = []
    xml_files = glob.iglob(os.path.join(source_folder, '*.xml'))
    for xml_file in xml_files:
        forum_name = os.path.basename(xml_file)[:-4]
        forum_names.append(forum_name)
        print('Processing {}'.format(forum_name), flush=True)
        forum_text = get_forum_text(xml_file)
        forum_texts.append(forum_text)
        word_count = Counter(forum_text)
        word_counts.append(word_count)
        write_forum_counts(forum_name, word_count, data_folder)
    write_corpus_counts(source_folder, word_counts, data_folder)
    significances = find_significances(forum_texts, word_counts)
    write_significances(forum_names, significances, reports_folder)


def get_forum_text(xml_file):
    """Return post texts from forum xml file."""
    tree = ET.parse(xml_file)
    root = tree.getroot()
    posts = root.iter('post')
    texts = [post.find('message').text for post in posts
             if is_current(post)]
    prepared_text = prepare_text(texts)
    return prepared_text


def prepare_text(forum_text):
    """Return content words of all message texts in one list."""
    prepared_text = []
    for post in forum_text:
        post_text = STRIP_TAGS(' ', html.unescape(post))
        words = EXTRACT_WORDS(post_text.casefold())
        stop_words = get_stop_words(STOP_FILE)
        pruned_words = [word for word in words
                        if (word not in stop_words and
                            len(word) > 2 and
                            word.isalpha())]
        prepared_text.extend(pruned_words)
    return prepared_text


def write_forum_counts(forum_name, word_count, data_folder):
    """Write word counts for each forum to csv."""
    counts_file = os.path.join(data_folder, forum_name + '_words_counts.csv')
    with open(counts_file, 'wt', encoding='utf-8', newline='\n') as csv_counts:
        counts_writer = csv.writer(csv_counts)
        counts_writer.writerow(['Word', 'Count'])
        counts_writer.writerows(word_count.most_common())


def write_corpus_counts(source_folder, word_counts, data_folder):
    """Write word counts for entire folder to csv."""
    corpus_counts = Counter()
    for word_count in word_counts:
        corpus_counts.update(word_count)
    clean_name = (source_folder
                  .replace('.', '-')
                  .replace('/', '-')
                  .replace('\\', '-'))
    counts_file = os.path.join(data_folder, clean_name + '_words_counts.csv')
    with open(counts_file, 'wt', encoding='utf-8', newline='\n') as csv_counts:
        counts_writer = csv.writer(csv_counts)
        counts_writer.writerow(['Word', 'Count'])
        counts_writer.writerows(corpus_counts.most_common())


def find_significances(forum_texts, word_counts):
    """Calculate significance values for words in each forum."""
    def _tf(word_counts, i, word, total_words):
        return word_counts[i][word] / total_words

    def _n_containing(word, word_counts):
        return sum(1 for word_count in word_counts if word in word_count)

    def _idf(total_forum_texts, word, word_counts):
        return (math.log(total_forum_texts) /
                (1 + _n_containing(word, word_counts)))

    def _tfidf(word_counts, i, word, total_words, total_forum_texts):
        return (_tf(word_counts, i, word, total_words) *
                _idf(total_forum_texts, word, word_counts))

    significances = []
    total_forum_texts = len(forum_texts)
    for i, forum_text in enumerate(forum_texts):
        total_words = len(forum_text)
        scores = {word: _tfidf(word_counts, i, word, total_words, total_forum_texts)
                  for word in forum_text}
        sorted_words = sorted(scores.items(), key=itemgetter(1), reverse=True)
        significances.append(sorted_words)
    return significances


def write_significances(forum_names, significances, reports_folder):
    """Write html report listing word significance for each forum."""
    for i, forum_name in enumerate(forum_names):
        results_file = os.path.join(reports_folder,
                                    '{}_significant_words.html'
                                    .format(forum_name))
        with open(results_file, 'wt', encoding='utf-8', newline='\n') as fout:
            fout.write(PAGE_HEADER.format('Suggested keywords for ' +
                                          forum_name))
            fout.write(TABLE_HEADER)
            formatted_rows = (TABLE_ROW.format(word, score)
                              for word, score in significances[i])
            for formatted_row in formatted_rows:
                fout.write(formatted_row)
            fout.write('</table>\n' + PAGE_FOOTER)


if __name__ == '__main__':
    sys.exit(suggest_keywords('../forums/'))
    # sys.exit(suggest_keywords('../forums/',
                              # data_folder='../data-test/',
                              # reports_folder='../reports-test/'))
