#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Track classification of posts during a presentation.

:author: `David King <mailto:david.king@open.ac.uk>`_
:project: eSTEeM - Measuring Qualification Effects framework
:copyright: `2016 The Open University <http://www.open.ac.uk>`_
:licence: `GPL v3 <http://www.gnu.org/licenses/>`_
"""

from collections import Counter, defaultdict
import csv
from datetime import date, timedelta
import glob
from operator import itemgetter
import os
import pickle
import re
import sys

FIND_PRESENTATION = re.compile(r'''
[A-Z]+\d{3}-
(?P<presentation>\d{2}[A-M])
\s
''', re.VERBOSE).search


def get_forum_posts_data(source_folder):
    """Return forum name and data from each file in source folder."""
    print('Processing posts files in {}'.format(source_folder),
          flush=True)
    forum_files = glob.iglob(os.path.join(source_folder, '*.pkl'))
    for forum_file in forum_files:
        forum_name = os.path.basename(forum_file)[:-4]
        with open(forum_file, 'rb') as fin:
            yield (forum_name, pickle.load(fin))


def set_results_folder(results_folder):
    """Ensure we have somewhere to write."""
    if not os.path.exists(results_folder):
        os.mkdir(results_folder)


def load_taxonomy(taxon_set, classifiers_folder):
    """Return taxonomy to classify forum posts."""
    taxon_file = os.path.join(classifiers_folder, taxon_set + '.pkl')
    with open(taxon_file, 'rb') as fin:
        return pickle.load(fin)


def track_classified_posts(source_folder,
                           classifiers_folder='../classifiers',
                           data_folder='../data/',
                           taxon_set='bloom'):
    """Submit for processing all pickled files in source folder."""
    set_results_folder(data_folder)
    taxon_keys = load_taxonomy(taxon_set, classifiers_folder).keys
    for forum_name, forum_posts_data in get_forum_posts_data(source_folder):
        print('Processing {}'.format(forum_name), flush=True)
        forum_posts_data.sort(key=itemgetter('created'))
        posts_file = os.path.join(data_folder,
                                  forum_name + '_classified_posts.csv')
        write_posts(forum_posts_data, posts_file,
                    taxon_set, taxon_keys)
        daily_file = os.path.join(data_folder,
                                  forum_name + '_daily_counts.csv')
        classified_file = os.path.join(data_folder,
                                       forum_name + '_classified_counts.csv')
        write_counts(forum_posts_data, daily_file, classified_file,
                     taxon_set, taxon_keys)


def write_posts(forum_data, posts_file, taxon_set, taxon_keys):
    """Write metadata for selected posts to csv."""
    with open(posts_file, 'wt', encoding='utf-8', newline='\n') as csv_posts:
        posts_writer = csv.writer(csv_posts)
        header_row = ['PostID',
                      'UserID',
                      'Date'] + [key.capitalize()
                                 for key in taxon_keys()]
        posts_writer.writerow(header_row)
        for post in forum_data:
            body_row = [post['postid'],
                        post['userid'],
                        post['created']] + [post[taxon_set][key]
                                            for key in taxon_keys()]
            posts_writer.writerow(body_row)


def get_daily_posts(forum_data):
    """Return list of number of posts for each day."""
    createds = [post['created'] for post in forum_data]
    count_createds = Counter(createds)
    return sorted([(created, count) for created, count
                   in count_createds.items()])


def write_counts(forum_data, daily_file, classified_file,
                 taxon_set, taxon_keys):
    """Write daily counts of selected posts to csv."""
    keys = sorted([key for key in taxon_keys()])  # sort to ensure consistency
    with open(daily_file, 'wt',
              encoding='utf-8', newline='\n') as csv_daily, \
        open(classified_file, 'wt',
             encoding='utf-8', newline='\n') as csv_counts:
        daily_writer = csv.writer(csv_daily)
        daily_writer.writerow(['Date',
                               'Count'])
        counts_writer = csv.writer(csv_counts)
        counts_writer.writerow(['Date'] +
                               ['{} Posts'.format(key.capitalize())
                                for key in keys] +
                               ['{} Use'.format(key.capitalize())
                                for key in keys])
        daily_posts = get_daily_posts(forum_data)
        if not daily_posts:  # forum has no posts
            return
        daily_counts = defaultdict(dict)
        for created, count in daily_posts:
            daily_counts[created]['all_posts'] = count
            for key in keys:
                daily_counts[created][key] = [0, 0]  # list [0] Posts [1] Use
        for post in forum_data:
            post_date = post['created']
            for key in keys:
                if post[taxon_set][key]:
                    daily_counts[post_date][key][0] += 1
                    daily_counts[post_date][key][1] += post[taxon_set][key]
        next_day = timedelta(days=1)
        presentation = FIND_PRESENTATION(daily_file).group('presentation')
        start_year = int('20' + presentation[:2])  # cast for date()
        if presentation[2] == 'E':  # E -> Apr-Oct
            current_date = date(start_year, 4, 1)
            end_date = date(start_year, 10, 31)
        elif presentation[2] == 'K':  # K -> Oct-Apr
            current_date = date(start_year, 10, 1)
            end_date = date(start_year + 1, 4, 30)
        first_post = daily_posts[0]
        last_post = daily_posts[-1]
        if daily_counts:
            if first_post[0] < current_date:  # first post before forum opens
                current_date = first_post[0]
            if last_post[0] > end_date:  # last post after course ends
                end_date = last_post[0]
        for post_date in sorted(daily_counts.keys()):
            while post_date != current_date:
                daily_writer.writerow([current_date, 0])
                counts_writer.writerow([current_date] + [0] * len(keys) * 2)
                current_date += next_day
            current_date += next_day
            daily_writer.writerow([post_date,
                                   daily_counts[post_date]['all_posts']])
            counts_writer.writerow([post_date] +
                                   [daily_counts[post_date][key][0]
                                    for key in keys] +
                                   [daily_counts[post_date][key][1]
                                    for key in keys])
        while current_date <= end_date:
            daily_writer.writerow([current_date, 0])
            counts_writer.writerow([current_date] + [0] * len(keys) * 2)
            current_date += next_day


if __name__ == '__main__':
    # sys.exit(track_classified_posts('../posts-classified-solo/',
                                    # data_folder='../data-solo/',
                                    # taxon_set='solo'))
    sys.exit(track_classified_posts('../posts-classified/'))
