#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Track posts that contain external references during a presentation.

:author: `David King <mailto:david.king@open.ac.uk>`_
:project: eSTEeM - Measuring Qualification Effects framework
:copyright: `2016 The Open University <http://www.open.ac.uk>`_
:licence: `GPL v3 <http://www.gnu.org/licenses/>`_
"""

from collections import Counter
import csv
from datetime import date
from datetime import timedelta
import glob
from operator import itemgetter
import os
import re
import sys
import xml.etree.ElementTree as ET

FIND_CITATION = re.compile(r'\[\d+\]').search
FIND_HTTP = re.compile(r'https?\:').search
FIND_PRESENTATION = re.compile(r'''
[A-Z]+\d{3}-
(?P<presentation>\d{2}[A-M])
\s
''', re.VERBOSE).search
FIND_REFERENCES = re.compile(r'\breferences\b', re.IGNORECASE).search
FIND_YEAR_REF = re.compile(r'\(\d{4}\)').search


def is_current(post):
    """Return True if post is latest version of that post in the forum."""
    return True if post.find('oldversion').text == '0' else False


def has_link(post):
    """Return True if post has an indicator of external link."""
    message_text = post.find('message').text
    return True if (FIND_HTTP(message_text) or
                    FIND_REFERENCES(message_text) or
                    FIND_YEAR_REF(message_text) or
                    FIND_CITATION(message_text)) else False


def get_forum_data(source_folder):
    """Return forum name and parsed XML from each file in source folder."""
    xml_files = glob.iglob(os.path.join(source_folder, '*.xml'))
    for xml_file in xml_files:
        forum_name = os.path.basename(xml_file)[:-4]
        tree = ET.parse(xml_file)
        root = tree.getroot()
        posts = root.iter('post')
        yield (forum_name, posts)


def set_results_folder(results_folder):
    """Ensure we have somewhere to write."""
    if not os.path.exists(results_folder):
        os.mkdir(results_folder)


def track_filtered_posts(source_folder, data_folder='../data/'):
    """Process all .xml files in source folder."""
    set_results_folder(data_folder)
    print('Processing forum posts files in {}'.format(source_folder),
          flush=True)
    for forum_name, posts in get_forum_data(source_folder):
        print('Processing {}'.format(forum_name), flush=True)
        write_metadata(get_posts_metadata(posts), forum_name, data_folder)


def get_posts_metadata(posts):
    """Return metadata for posts in forum."""
    posts_metadata = []
    for post in posts:
        post_metadata = get_post_metadata(post)
        if post_metadata:
            posts_metadata.append(post_metadata)
    posts_metadata.sort(key=itemgetter(2))  # sort on created
    return posts_metadata


def get_post_metadata(post):
    """Return metadata if post is selected."""
    if is_current(post) and has_link(post):
        postid = post.get('id')
        userid = post.find('userid').text
        created = date.fromtimestamp(int(post.find('created').text))
        return (postid, userid, created)


def write_metadata(posts_metadata, forum_name, data_folder):
    """Write forum posts' metadata to data folder."""
    posts_file = os.path.join(data_folder, forum_name + '_filtered_posts.csv')
    write_posts(posts_metadata, posts_file)
    counts_file = os.path.join(data_folder, forum_name + '_filtered_counts.csv')
    write_counts(posts_metadata, counts_file)


def write_posts(posts_metadata, posts_file):
    """Write metadata for selected posts to csv."""
    with open(posts_file, 'wt', encoding='utf-8', newline='\n') as csv_posts:
        posts_writer = csv.writer(csv_posts)
        posts_writer.writerow(['PostID', 'UserID', 'Date'])
        posts_writer.writerows(posts_metadata)


def get_counts(posts_metadata):
    """Return list of number of posts each day."""
    createds = [created for _, _, created in posts_metadata]
    count_createds = Counter(createds)
    return sorted([(created, count) for created, count in count_createds.items()])


def write_counts(posts_metadata, counts_file):
    """Write daily counts of posts to csv."""
    with open(counts_file, 'wt', encoding='utf-8', newline='\n') as csv_counts:
        counts_writer = csv.writer(csv_counts)
        counts_writer.writerow(['Date', 'Count'])
        counts = get_counts(posts_metadata) if posts_metadata else []
        next_day = timedelta(days=1)
        presentation = FIND_PRESENTATION(counts_file).group('presentation')
        start_year = int('20' + presentation[:2])  # cast for date()
        if presentation[2] == 'E':  # E -> Apr-Dec
            current_date = date(start_year, 4, 1)
            end_date = date(start_year, 12, 31)
        elif presentation[2] == 'K':  # K -> Oct-Jun
            current_date = date(start_year, 10, 1)
            end_date = date(start_year + 1, 6, 30)
        if counts:
            if counts[0][0] < current_date:  # first post before course starts
                current_date = counts[0][0]
            if counts[-1][0] > end_date:  # last post after course ends
                end_date = counts[-1][0]
        for created, count in counts:
            while created != current_date:
                counts_writer.writerow([current_date, 0])
                current_date += next_day
            current_date += next_day
            counts_writer.writerow([created, count])
        while current_date <= end_date:
            counts_writer.writerow([current_date, 0])
            current_date += next_day


if __name__ == '__main__':
    sys.exit(track_filtered_posts('../forums/'))
    # sys.exit(track_filtered_posts('../forums-test/',
             # data_folder='../data-test'))
