#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Identify unique userids for each days' posts in forums.

:author: `David King <mailto:david.king@open.ac.uk>`_
:project: eSTEeM - Measuring Qualification Effects framework
:copyright: `2016 The Open University <http://www.open.ac.uk>`_
:licence: `GPL v3 <http://www.gnu.org/licenses/>`_
"""

from collections import defaultdict
from collections import namedtuple
import csv
import glob
from operator import attrgetter
import os
import sys

DailyData = namedtuple('Daily_data', 'created, posts, users, userids')

PAGE_HEADER = '''<!DOCTYPE html>
<html>
<head>
<meta charset="utf-8">
<title>{}</title>
</head>
<body>
'''

PAGE_FOOTER = '''</body>
</html>
'''

TABLE_HEADER = '''<table>
<tr>
<th align="left">Date</th>
<th align="left">Posts</th>
<th align="left">Users</th>
<th align="left">UserIDs</th>
</tr>
'''

TABLE_ROW = '''<tr>
<td>{}</td>
<td align="right">{}</td>
<td align="right">{}</td>
<td>{}</td>
</tr>
'''


def set_results_folder(results_folder):
    """Ensure we have somewhere to write."""
    if not os.path.exists(results_folder):
        os.mkdir(results_folder)


def unique_userids(source_folder,
                   data_folder='../data/',
                   reports_folder='../reports/'):
    """Submit for processing all *_posts.csv files in source folder."""
    set_results_folder(data_folder)
    set_results_folder(reports_folder)
    for forum_name, forum_usage in get_forum_usage(source_folder):
        write_usage_report(forum_name, forum_usage, reports_folder)
        write_usage_table(forum_name, forum_usage, data_folder)


def get_forum_usage(source_folder):
    """Return forum name and usage from each file in source folder."""
    print('Processing forum posts files in {}'.format(source_folder),
          flush=True)
    csv_files = glob.iglob(os.path.join(source_folder, '*_classified_posts.csv'))
    for csv_file in csv_files:
        forum_name = os.path.basename(csv_file)[:-21]
        print('Processing {}'.format(forum_name), flush=True)
        forum_usage = prepare_forum_usage(get_users_each_day(csv_file))
        yield (forum_name, forum_usage)


def prepare_forum_usage(userids_by_created):
    """Return comprehensive daily usage data for a forum."""
    forum_usage = []
    for created, userids in userids_by_created.items():
        posts = len(userids)
        userids = tuple(sorted(set(userids)))
        users = len(userids)
        forum_usage.append(DailyData(created=created,
                                     posts=posts,
                                     users=users,
                                     userids=userids))
    forum_usage.sort(key=attrgetter('created'))
    return forum_usage


def get_users_each_day(posts_file):
    """Return daily lists of userids that have posted to a forum."""
    with open(posts_file, 'rt', encoding='utf-8', newline='\n') as csv_posts:
        posts_reader = csv.DictReader(csv_posts)
        userids_by_created = defaultdict(list)
        for post in posts_reader:
            formatted_userid = '{:0>6}'.format(post['UserID'])
            userids_by_created[post['Date']].append(formatted_userid)
    return userids_by_created


def write_usage_report(forum_name, forum_usage, reports_folder):
    """Write usage as a table in a web page."""
    output_file = os.path.join(reports_folder, forum_name + '_usage.html')
    with open(output_file, 'wt', encoding='utf-8', newline='\n') as fout:
        fout.write(PAGE_HEADER.format('Daily usage'))
        fout.write(TABLE_HEADER)
        for daily_data in forum_usage:
            fout.write(TABLE_ROW.format(daily_data.created,
                                        daily_data.posts,
                                        daily_data.users,
                                        ', '.join(daily_data.userids)))
        fout.write('</table>\n' + PAGE_FOOTER)


def write_usage_table(forum_name, forum_usage, data_folder):
    """Write usage as a table to csv."""
    output_file = os.path.join(data_folder, forum_name + '_usage.csv')
    with open(output_file, 'wt', encoding='utf-8', newline='\n') as csv_output:
        csv_writer = csv.writer(csv_output)
        csv_writer.writerow(['Date', 'Posts', 'Users', 'UserIDs'])
        for daily_data in forum_usage:
            csv_writer.writerow([daily_data.created,
                                 daily_data.posts,
                                 daily_data.users,
                                 ' '.join(daily_data.userids)])


if __name__ == '__main__':
    sys.exit(unique_userids('../data/'))
    # sys.exit(unique_userids('../data-test/',
                            # data_folder='../data-test/',
                            # reports_folder='../reports-test/'))
